
' =======
' See ShowUsage() for details.
' =======

Option Explicit

' Variables used by command line arguments and help page display
Private strArg, objArgs, strUsage, strParam

' Variables for main functions
Const ForReading = 1, ForWriting = 2, ForAppending = 8
Private strTargetPath, strSourcePath, strRegPath, strMessage
Private bDoneSomething
strTargetPath = "C:\Temp"
strSourcePath = ".\"
bDoneSomething = false

' Variables for ini file update
Private strIniFileName, strIniFilePath, strTargetIni
Private strIniSectionName, strIniLineToAdd, strEnterAfterLine
Private strBackupFile
strIniFileName = "\System\UT2004.ini"   ' see also the UT2003.ini section
strIniSectionName = "[Engine.GameEngine]"
strIniLineToAdd = "ServerPackages=jcbContentMania"
strEnterAfterLine = "ServerPackages="

' Variables for the log file
Private strLogFileName, strLogEntry
strLogFileName = ".\SetupMod.log"

On Error Resume Next

If bEvalArgs() Then

  strLogEntry = "*** Start Installation..." & vbCrLf & "=========================" 
  AppendToLogFile(strLogEntry)

  PromptThenCopyFiles()
  PromptThenEditIni()

  CheckForError("Main Section")
  strLogEntry = "!!! Installation cancelled !!!" & vbCrLf 
  If bDoneSomething Then
    strLogEntry = "*** Installation complete ***" & vbCrLf 
  End If
  AppendToLogFile(strLogEntry)
  WScript.Echo strLogEntry

End If	' bEvalArgs() is False
' *** Execution should end here ***

Sub PromptThenEditIni()
  On Error Resume Next

  ' ** Add server entry to .ini file
  strMessage = "SERVER SETTING" & vbCrLf & vbCrLf & _
  "To use this mod for multiplayer games the following line " & _
  "must be included in the server .ini file in the section:" & vbCrLf & _
  "" & vbCrLf & _
  strIniSectionName & vbCrLf & _
  "..." & vbCrLf & _
  strIniLineToAdd & vbCrLf & _
  "" & vbCrLf & _
  "If you want this line added for you, enter the full path and name of the .ini " & _
  "file to edit." & vbCrLf & _
  "You can add this even if you do not host a server." & vbCrLf
  
  strIniFilePath = strRegPath & strIniFileName

  strTargetIni = InputBox(strMessage, "Unreal Tournament INI File", strIniFilePath )
  strLogEntry = "Target INI file = " & strTargetIni
  AppendToLogFile(strLogEntry)
  If strTargetIni <> "" Then
    If bBackupIniFile(strTargetIni) Then
      If bUpdateIni() Then
        bDoneSomething = true
      End If
    End If
  Else
    strLogEntry = "Update INI file cancelled!"
    AppendToLogFile(strLogEntry)
  End If

  CheckForError("PromptThenEditIni")
End Sub

Function bUpdateIni()
  On Error Resume Next
  Dim bReturn
  bReturn = false
  
  If strTargetIni <> "" Then
      bReturn = bWriteToIniFile()
  Else
    strLogEntry = "INI file not entered!"
    AppendToLogFile(strLogEntry)
  End If
  
  CheckForError("bUpdateIni")
  bUpdateIni = bReturn
End Function

' Write data to the ini file
Function bWriteToIniFile()
  On Error Resume Next
  Dim bReturn, bFoundSection, bFoundLineType
  Dim objFileSys, objFileOld, objFileNew
  Dim strData, strTestData
  bReturn = false
  bFoundSection = false
  bFoundLineType = false
  Set objFileSys = CreateObject("Scripting.FileSystemObject")

  If objFileSys.FileExists(strBackupFile) Then
    
    If Not objFileSys.FileExists(strTargetIni) Then
       objFileSys.CreateTextFile(strTargetIni)    ' create the ini if it does not already exist
    End If
  
    ' Read from the backup file, Write to the original
    If objFileSys.FileExists(strTargetIni) Then
      
      Set objFileOld = objFileSys.OpenTextFile(strBackupFile, ForReading)
      Set objFileNew = objFileSys.OpenTextFile(strTargetIni, ForWriting, false) ' ASCII
      
      While not objFileOld.AtEndOfStream
        strData = objFileOld.ReadLine
        
        If LCase(Trim(strData)) = LCase(Trim(strIniSectionName)) Then
          strLogEntry = "INI Section Name Found = " & strData
          AppendToLogFile(strLogEntry)
          bFoundSection = true
        End If
        
        strTestData = LCase(Left(Trim(strData),Len(strEnterAfterLine)))   ' first part of the line

        If bFoundSection and strTestData = LCase(strEnterAfterLine) Then
          bFoundSection = false   ' stop it adding more than once
          bFoundLineType = true
        End If
        
        If bFoundLineType and LCase(Trim(strData)) = LCase(Trim(strIniLineToAdd)) Then
          ' stop duplicate lines
          strLogEntry = "Line already included in INI file = " & strData
          AppendToLogFile(strLogEntry)
          bFoundSection = false   ' stop it adding again
          bFoundLineType = false   ' stop it adding again
        End if
        
        If bFoundLineType and strTestData <> LCase(strEnterAfterLine) Then
          ' we must be at the line we want to enter in to
          objFileNew.WriteLine strIniLineToAdd
          strLogEntry = "Added line to INI file = " & strIniLineToAdd
          AppendToLogFile(strLogEntry)
          bFoundSection = false   ' stop it adding again
          bFoundLineType = false   ' stop it adding again
        End if
        
        objFileNew.WriteLine strData
      Wend
      
      objFileOld.Close
      objFileNew.Close
      
      bReturn = true

    Else
      strLogEntry = "INI file not created!"
      AppendToLogFile(strLogEntry)
    End If
  Else
    strLogEntry = "Backup INI file does not exist!"
    AppendToLogFile(strLogEntry)
  End If  ' backup file exists

  CheckForError("bWriteToIniFile")
  bWriteToIniFile = bReturn
End Function

Function bBackupIniFile(byVal strSourceFile)
  On Error Resume Next
  Dim bReturn
  Dim objFileSys

  bReturn = false
  Set objFileSys = CreateObject("Scripting.FileSystemObject")

  If objFileSys.FileExists(strSourceFile) Then
    strBackupFile = strGetBackupName(1, strSourceFile)
    If strBackupFile <> "" Then
      If objFileSys.FileExists(strBackupFile) Then
        strLogEntry = "Backup INI File Already Exists..."
        AppendToLogFile(strLogEntry)
      Else
        objFileSys.CopyFile strSourceFile, strBackupFile
        bReturn = true
      End If
    End If
    strLogEntry = "Backup INI File = " & strBackupFile
    AppendToLogFile(strLogEntry)
  Else
    strLogEntry = "INI File does not exist = " & strSourceFile
    AppendToLogFile(strLogEntry)
  End If
  

  CheckForError("bBackupIniFile")
  bBackupIniFile = bReturn
End Function

Function strGetBackupName(byVal iFileCount, strFileName)
  On Error Resume Next
  Dim strReturn
  Dim objFileSys
  strReturn = ""


  If iFileCount < 1 Then
    iFileCount = 1
  End If

  Set objFileSys = CreateObject("Scripting.FileSystemObject")
  strReturn = strFileName & "-bak" & CStr(iFileCount)
  If objFileSys.FileExists(strReturn) Then
    iFileCount = iFileCount + 1 ' keep adding one until we find an unused name
    strReturn = strGetBackupName(iFileCount, strFileName)
  End If
  
  CheckForError("strGetBackupName")
  strGetBackupName = strReturn
End Function

Sub PromptThenCopyFiles()
  On Error Resume Next

  strRegPath = ""
  ' UT2004
  strRegPath = strGetRegVal("HKEY_LOCAL_MACHINE\SOFTWARE\Unreal Technology\Installed Apps\UT2004\Folder")
  If strRegPath = "" Then
    ' try UT2003
    strRegPath = strGetRegVal("HKEY_LOCAL_MACHINE\SOFTWARE\Unreal Technology\Installed Apps\UT2003\Folder")
    strIniFileName = "\System\UT2003.ini"
  End if
  strLogEntry = "Unreal Location = " & strRegPath
  AppendToLogFile(strLogEntry)
  If strRegPath <> "" Then

    strMessage = "INSTALL PATH" & vbCrLf & vbCrLf & _
    "To install the mod, enter the base folder for the version of Unreal " & _
    "Tournament you want to add it to." & vbCrLf & _
    "The folder is the one selected when originally installing the game." & vbCrLf & _
    "If you have more than one compatible version of Unreal Tournament installed, " & _
    "you can re-run this setup script and enter another folder." & vbCrLf

    strTargetPath = InputBox(strMessage, "Unreal Tournament Install Path", strRegPath )
    strLogEntry = "Target path = " & strTargetPath
    AppendToLogFile(strLogEntry)
    If strTargetPath <> "" Then
      CopyFolders (strSourcePath)
      bDoneSomething = true
    End If
  End If  ' Path not blank
  CheckForError("PromptThenCopyFiles")

End Sub

Function strGetRegVal( strRegKey )
  On Error Resume Next
  strGetRegVal = ""
  Dim objWShell, strTestIn, strRegVal
  Set objWShell = WScript.CreateObject("WScript.Shell")
  strGetRegVal = objWShell.RegRead(strRegKey)
  CheckForError()
End Function

Sub CopyFolders (ByVal strContainerPath)
  On Error Resume Next
  Dim objFileSys, folContainer, folSubFolders, folSubFolder, bContinue
  Dim strSourceFolder

  bContinue = true

  Set objFileSys = CreateObject("Scripting.FileSystemObject")
  Set folContainer = objFileSys.GetFolder(strContainerPath)
  Set folSubFolders = folContainer.SubFolders

  strSourceFolder = folContainer.Path

  If strTargetPath = strSourceFolder Then
    bContinue = false
    strLogEntry = "Source and destination the same!" & vbCrLf & _
      " Source = " & strSourceFolder & vbCrLf & _
      " Target = " & strTargetPath & vbCrLf & _
      "Copy aborted!" 
    AppendToLogFile(strLogEntry)
  End If

  If bContinue Then
    strLogEntry = "Source folder path = " & vbCrLf & strSourceFolder & vbCrLf & _
      "Start copying folders ..." 
    AppendToLogFile(strLogEntry)
  
    For Each folSubFolder in folSubFolders
      if objFileSys.FolderExists(folSubFolder.Path) then
         AppendToLogFile(folSubFolder.Path)
         objFileSys.CopyFolder folSubFolder.Path , strTargetPath & "\" , true
         CheckForError()
      end if
    Next
  
    CheckForError()
    strLogEntry = "End copying folders."
    AppendToLogFile(strLogEntry)
  End If  ' bContinue

  Set objFileSys = Nothing
  Set folContainer = Nothing
  Set folSubFolders = Nothing
End Sub

' Write data to the log file
Sub AppendToLogFile(strData)
  On Error Resume Next

  Dim objFileSys
  Set objFileSys = CreateObject("Scripting.FileSystemObject")

  if Not objFileSys.FileExists(strLogFileName) then
     objFileSys.CreateTextFile(strLogFileName)
  end if

  ' This opens and closes the log file for each write 
  ' if it is used a lot write another function
  With objFileSys.OpenTextFile(strLogFileName, ForAppending)
   '.Write strData & vbCrLf: .Close
   .WriteLine strData: .Close
  End With 
End Sub


' Used for checking installed version of WSH is compatible
Sub ShowVersions()
  WScript.Echo "WSH Version: " & WScript.Version & vbCrLf & _
  "VBScript Version: " & ScriptEngineMajorVersion _
      & "." & ScriptEngineMinorVersion
End Sub

Sub CheckForError(strSectionRef)
  If strSectionRef = "" Then
    strSectionRef = "Number"
  End If
  If Err.Number > 0 Then
   strLogEntry = "* Error " & strSectionRef & " - " & CStr(Err.Number) & " " & Err.Description & " *" & vbCrLf 
   AppendToLogFile(strLogEntry)
   Err.Clear
  End If
End Sub

' For all scripts to show options and usage
Function bEvalArgs()
  Set objArgs = WScript.Arguments
  bEvalArgs = True	' continue the script
  For each strArg in objArgs
	  If Len(strArg) > 1 Then
	    strParam = LCase(Left(strArg,2))
	    Select Case strParam
		    Case "/t"
		    	 If Len(strArg) > 3 Then
		    		strTargetPath = LCase(Right(strArg,Len(strArg) - 3))
			     End If
		    Case "/v"
          ShowVersions
		      bEvalArgs = False	' halt the script
		    Case "/?"
		      ShowUsage
		      bEvalArgs = False	' halt the script
	    End Select
	  End If
  Next
End Function

Sub ShowUsage()

	strUsage = "SetupMod.vbs " & vbCrLf & _
		"wscript //nologo SetupMod.vbs [/v|/? /t=path] " & vbCrLf & _
		" " & vbCrLf & _
		"Version 32a - May 2004 - Author: John C Brown " & vbCrLf & _
		" " & vbCrLf & _
		"Simple Application Setup " & vbCrLf & _
		"======================== " & vbCrLf & _
		"Copies folders and their contents from the current location " & vbCrLf & _
		"to a destination, either read from a registry setting or the command " & vbCrLf & _
		"line.  Files within the same folder as this script are not copied. " & vbCrLf & _
		" " & vbCrLf & _
		"This is designed to replace complicated installation programmes for the " & vbCrLf & _
		"distribution of patches, updates or modifications. " & vbCrLf & _
		" " & vbCrLf & _
		"Include all the files and folders in a simple zip file including this script. " & vbCrLf & _
    "The script then guides the end user in putting the files in to " & vbCrLf & _
		"the correct location. " & vbCrLf & _
		" " & vbCrLf & _
		"Parameter Options: " & vbCrLf & _
		"/t=path Specify the target folder " & vbCrLf & _
		"/v  Show the WSH and VB versions installed " & vbCrLf & _
		"/?  Help page (this) " & vbCrLf & _
		" " & vbCrLf

	Wscript.Echo strUsage

End Sub
